Package: reactable
Type: Package
Title: Interactive Data Tables for R
Version: 0.4.5
Authors@R: c(
    person("Greg", "Lin", email = "glin@glin.io", role = c("aut", "cre")),
    person("Tanner", "Linsley", role = c("ctb", "cph"), comment = "React Table library"),
    person(family = "Emotion team and other contributors", role = c("ctb", "cph"),
           comment = "Emotion library"),
    person("Kent", "Russell", role = c("ctb", "cph"), comment = "reactR package"),
    person("Ramnath", "Vaidyanathan", role = c("ctb", "cph"), comment = "htmlwidgets package"),
    person("Joe", "Cheng", role = c("ctb", "cph"), comment = "htmlwidgets package"),
    person("JJ", "Allaire", role = c("ctb", "cph"), comment = "htmlwidgets package"),
    person("Yihui", "Xie", role = c("ctb", "cph"), comment = "htmlwidgets package"),
    person("Kenton", "Russell", role = c("ctb", "cph"), comment = "htmlwidgets package"),
    person(family = "Facebook, Inc. and its affiliates", role = c("ctb", "cph"), comment = "React library"),
    person(family = "FormatJS", role = c("ctb", "cph"), comment = "FormatJS libraries"),
    person(family = "Feross Aboukhadijeh, and other contributors", role = c("ctb", "cph"),
           comment = "buffer library"),
    person("Roman", "Shtylman", role = c("ctb", "cph"), comment = "process library"),
    person("James", "Halliday", role = c("ctb", "cph"), comment = "stream-browserify library"),
    person(family = "Posit Software, PBC", role = c("fnd", "cph"))
    )
Description: Interactive data tables for R, based on the 'React Table'
    JavaScript library. Provides an HTML widget that can be used in 'R Markdown'
    or 'Quarto' documents, 'Shiny' applications, or viewed from an R console.
License: MIT + file LICENSE
URL: https://glin.github.io/reactable/,
        https://github.com/glin/reactable
BugReports: https://github.com/glin/reactable/issues
Depends: R (>= 3.1)
Imports: digest, htmltools (>= 0.5.2), htmlwidgets (>= 1.5.3),
        jsonlite, reactR
Suggests: covr, crosstalk, dplyr, fontawesome, knitr, leaflet, MASS,
        rmarkdown, shiny, sparkline, testthat, tippy, V8
Encoding: UTF-8
RoxygenNote: 7.2.1
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2025-12-01 15:17:46 UTC; greg
Author: Greg Lin [aut, cre],
  Tanner Linsley [ctb, cph] (React Table library),
  Emotion team and other contributors [ctb, cph] (Emotion library),
  Kent Russell [ctb, cph] (reactR package),
  Ramnath Vaidyanathan [ctb, cph] (htmlwidgets package),
  Joe Cheng [ctb, cph] (htmlwidgets package),
  JJ Allaire [ctb, cph] (htmlwidgets package),
  Yihui Xie [ctb, cph] (htmlwidgets package),
  Kenton Russell [ctb, cph] (htmlwidgets package),
  Facebook, Inc. and its affiliates [ctb, cph] (React library),
  FormatJS [ctb, cph] (FormatJS libraries),
  Feross Aboukhadijeh, and other contributors [ctb, cph] (buffer library),
  Roman Shtylman [ctb, cph] (process library),
  James Halliday [ctb, cph] (stream-browserify library),
  Posit Software, PBC [fnd, cph]
Maintainer: Greg Lin <glin@glin.io>
Repository: CRAN
Date/Publication: 2025-12-01 16:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 15:51:26 UTC; unix
