# Generated by roxygen2: do not edit by hand

S3method(as.tags,reactable)
export(JS)
export(colDef)
export(colFormat)
export(colGroup)
export(getReactableState)
export(knit_print.reactable)
export(reactable)
export(reactableLang)
export(reactableOutput)
export(reactableTheme)
export(renderReactable)
export(updateReactable)
import(htmltools)
import(htmlwidgets)
importFrom(htmlwidgets,JS)
