---
output:
  html_document:
    theme: null
    highlight: null
    mathjax: null
    self_contained: false
---

```{r, include=FALSE}
options(reactable.static = TRUE)
```

```{r, echo=FALSE}
library(reactable)

reactable(iris)
```
