---
title: "Shiny Demo"
output: html_document
---

```{r echo=FALSE}
library(htmltools)

tags$div(class = "iframe-container", tags$iframe(src = "https://glin.shinyapps.io/reactable"))
```

```{css echo=FALSE}
.iframe-container {
  display: flex;
  flex-direction: column;
  margin-top: 1rem;
  height: 100%;
  overflow: hidden;
}

.iframe-container iframe {
  width: 100%;
  height: 100%;
  border: 0;
}

/* rmarkdown html documents */
.main-container {
  padding: 0;
  max-width: 100%;
  height: 100vh;
}

body {
  overflow-y: hidden;
}

#header {
  display: none;
}

/* pkgdown articles */
.template-article {
  width: 100%;
  height: 100%;
}

.row {
  height: 100%;
}

.row > main {
  max-width: 100%;
}

.page-header {
  display: none;
}
```
